@@ -198,12 +198,59 @@ get_environment (int *argc, char ***argv, int *envc, char ***envv)
 	g_list_free (envar);
 }
 
+static void
+launch_selected (GtkTreeModel *model,
+		 GtkTreeIter  *iter,
+		 GtkWidget    *dialog)
+{
+	GnomeDesktopItem *ditem;
+	GError           *error = NULL;
+	GtkToggleButton  *terminal;
+	char             *name;
+
+	gtk_tree_model_get (model, iter, COLUMN_NAME, &name, -1);
+
+	if (!name)
+		return;
+                        
+	ditem = gnome_desktop_item_new_from_uri (name,
+						 GNOME_DESKTOP_ITEM_LOAD_NO_TRANSLATIONS,
+						 &error);
+
+	g_free (name);
+
+	if (!ditem) {
+		panel_error_dialog ("failed_to_load_desktop",
+				    _("Failed to load this program!\n%s"),
+				    error->message);
+		g_clear_error (&error);
+		return;
+	}
+
+        terminal = GTK_TOGGLE_BUTTON (
+			g_object_get_data (G_OBJECT (dialog), "terminal"));
+
+	/* Honor "run in terminal" button */
+	gnome_desktop_item_set_boolean (ditem,
+					GNOME_DESKTOP_ITEM_TERMINAL,
+					terminal->active);
+
+	if (!gnome_desktop_item_launch (ditem, NULL, 0, &error)) {
+		panel_error_dialog ("failed_to_load_desktop",
+				    _("Failed to load this program!\n%s"),
+				    error->message);
+		g_clear_error (&error);
+		return;
+	}
+
+	gnome_desktop_item_unref (ditem);
+}
+
 static void 
 run_dialog_response (GtkWidget *w, int response, gpointer data)
 {
 	GtkEntry *entry;
         GtkWidget *list;
-	GtkToggleButton *terminal;
 	char **argv = NULL;
 	char **temp_argv = NULL;
 	int argc, temp_argc;
@@ -227,15 +274,11 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 	}
         
         list = g_object_get_data (G_OBJECT (run_dialog), "dentry_list");
-        terminal = GTK_TOGGLE_BUTTON (g_object_get_data (G_OBJECT(w),
-							 "terminal"));
         
         if (g_object_get_data (G_OBJECT (run_dialog), "use_list")) {
-                char *name;
 		GtkTreeSelection *selection;
 		GtkTreeModel *model;
 		GtkTreeIter iter;
-		GValue value = {0, };
 
 		selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
 
@@ -244,48 +287,10 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 							&model, &iter))
 			return;
 
-		gtk_tree_model_get_value (model, &iter,
-					  COLUMN_NAME,
-					  &value);
-		name = g_strdup (g_value_get_string (&value));
-		g_value_unset (&value);
-
-                if (name != NULL) {
-			GError *error = NULL;
-                        GnomeDesktopItem *ditem;
-                        
-                        ditem = gnome_desktop_item_new_from_uri (name,
-								 GNOME_DESKTOP_ITEM_LOAD_NO_TRANSLATIONS,
-								 &error);
-			if (ditem != NULL) {
-                                /* Honor "run in terminal" button */
-				gnome_desktop_item_set_boolean (ditem,
-								GNOME_DESKTOP_ITEM_TERMINAL,
-								terminal->active);
-			}
-
-                        if (ditem == NULL) {
-                                panel_error_dialog ("failed_to_load_desktop",
-						    _("Failed to load this program!\n%s"),
-						    error->message);
-				g_clear_error (&error);
-			} else if ( ! gnome_desktop_item_launch (ditem,
-								 NULL /* file_list */,
-								 0 /* flags */,
-								 &error)) {
-                                panel_error_dialog ("failed_to_load_desktop",
-						    _("Failed to load this program!\n%s"),
-						    error->message);
-				g_clear_error (&error);
-			}
-
-			if (ditem != NULL) {
-				gnome_desktop_item_unref (ditem);
-			}
-
-			g_free (name);
-                }
+		launch_selected (model, &iter, w);
         } else {
+		GtkToggleButton *terminal;
+
                 entry = GTK_ENTRY (g_object_get_data (G_OBJECT (w), "entry"));
 
                 s = gtk_entry_get_text(entry);
@@ -294,10 +299,7 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
                         goto return_and_close;
 
                 /* evil eggies, do not translate! */
-                if (strcmp (s, "time shall be unixey") == 0) {
-                        foobar_widget_global_set_clock_format ("%s");
-                        goto return_and_close;
-                } else if (strcmp (s, "you shall bring us a shrubbery") == 0) {
+                if (strcmp (s, "you shall bring us a shrubbery") == 0) {
                         panel_info_dialog ("ni_ni_ni_ni",
 					   "NI! NI! NI! NI! NI! NI!");
                         goto return_and_close;
@@ -335,6 +337,9 @@ run_dialog_response (GtkWidget *w, int response, gpointer data)
 
                 get_environment (&temp_argc, &temp_argv, &envc, &envv);
 
+		terminal = GTK_TOGGLE_BUTTON (
+				g_object_get_data (G_OBJECT (w), "terminal"));
+
                 if (terminal->active) {
                         char **term_argv;
                         int term_argc;
@@ -623,21 +628,23 @@ static void
 entry_changed (GtkWidget *entry,
                gpointer   data)
 {
-        sync_entry_to_list (GTK_WIDGET (data));
+	if (run_dialog != NULL)
+		sync_entry_to_list (run_dialog);
 }
 
 /* Called when advanced contents are switched to or first shown */
 static void
-advanced_contents_shown (GtkWidget *vbox,
-                         GtkWidget *dialog)
+advanced_contents_shown (GtkWidget *vbox, gpointer data)
 {
         /* does nothing at the moment */
 }
 
 static void
-activate_run (GtkWidget *entry, GtkWidget *dialog)
+activate_run (GtkWidget *entry, gpointer data)
 {
-	gtk_dialog_response (GTK_DIALOG (dialog), PANEL_RESPONSE_RUN);
+	if (run_dialog != NULL)
+		gtk_dialog_response (GTK_DIALOG (run_dialog),
+				     PANEL_RESPONSE_RUN);
 }
 
 static GtkWidget*
@@ -668,17 +675,19 @@ create_advanced_contents (void)
 	g_signal_connect (G_OBJECT (entry), "destroy",
 			  G_CALLBACK (kill_completion),
 			  NULL);
+
+	gtk_tooltips_set_tip (panel_tooltips, entry, _("Executable name for selected application"), NULL);
  
         gtk_combo_set_use_arrows_always (GTK_COMBO (gentry), TRUE);
         g_object_set_data (G_OBJECT (run_dialog), "entry", entry);
 
 	g_signal_connect (G_OBJECT (entry), "activate",
-			    G_CALLBACK (activate_run),
-			    run_dialog);
+			  G_CALLBACK (activate_run),
+			  NULL);
         g_signal_connect (G_OBJECT (entry),
-                            "changed",
-                            G_CALLBACK (entry_changed),
-                            run_dialog);
+			  "changed",
+			  G_CALLBACK (entry_changed),
+			  NULL);
         
         w = gtk_button_new_with_mnemonic (_("_Browse..."));
         g_signal_connect(G_OBJECT(w), "clicked",
@@ -707,7 +716,7 @@ create_advanced_contents (void)
         g_signal_connect (G_OBJECT (vbox),
 			  "show",
 			  G_CALLBACK (advanced_contents_shown),
-			  run_dialog);
+			  NULL);
 
         return vbox;
 }
@@ -859,7 +868,9 @@ fill_list (GtkWidget *list)
 
 	if (add_icon_idle_id == 0)
 		add_icon_idle_id =
-			g_idle_add ((GSourceFunc) add_icon_idle, store);
+			g_idle_add_full (G_PRIORITY_LOW,
+					 (GSourceFunc)add_icon_idle,
+					 store, NULL);
 }
 
 #define DEFAULT_ICON "document-icons/i-executable.png"
@@ -940,14 +951,30 @@ unset_selected (GtkWidget *dialog)
 		(gtk_tree_view_get_selection (GTK_TREE_VIEW (list)));
 }
 
+static void
+selection_activated (GtkTreeView       *tree_view,
+		     GtkTreePath       *path,
+		     GtkTreeViewColumn *column,
+		     GtkWidget         *dialog)
+{
+	GtkTreeModel *model;
+	GtkTreeIter   iter;
+
+	model = gtk_tree_view_get_model (tree_view);
+	gtk_tree_model_get_iter (model, &iter, path);
+
+	launch_selected (model, &iter, dialog);
+
+	gtk_widget_destroy (dialog);
+}
+
 static void
 selection_changed (GtkTreeSelection *selection,
-		   gpointer data)
+		   GtkWidget        *dialog)
 {
         GtkWidget *label;
         GtkWidget *gpixmap;
         GtkWidget *desc_label;
-        GtkWidget *dialog = data;
         gchar *name;
 	GtkTreeModel *model;
 	GtkTreeIter iter;
@@ -1041,14 +1068,19 @@ create_simple_contents (void)
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (list), FALSE);
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label), list);
 
+	add_atk_name_desc (list, _("List of Applications"),
+			   _("Choose an application to run from the list"));
+	set_relation (list, GTK_LABEL (label), 0);
+
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (list));
 
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
 
-        g_signal_connect (G_OBJECT (selection),
-			  "changed",
-			  G_CALLBACK (selection_changed),
-			  run_dialog);
+        g_signal_connect (selection, "changed",
+			  G_CALLBACK (selection_changed), run_dialog);
+
+        g_signal_connect (list, "row-activated",
+			  G_CALLBACK (selection_activated), run_dialog);
         
         w = gtk_scrolled_window_new (NULL, NULL);
         gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (w),
@@ -1093,10 +1125,10 @@ create_simple_contents (void)
 				vbox,
 				(GtkDestroyNotify) g_object_unref);
 
-
-
 	if (add_items_idle_id == 0)
-		add_items_idle_id = g_idle_add (add_items_idle, list);
+		add_items_idle_id =
+			g_idle_add_full (G_PRIORITY_LOW, add_items_idle,
+					 list, NULL);
 
         gtk_box_pack_start (GTK_BOX (GTK_DIALOG (run_dialog)->vbox),
                             vbox,
@@ -1116,7 +1148,6 @@ update_contents (GtkWidget *dialog)
         GtkWidget *advanced = NULL;
         GtkWidget *advanced_toggle;
         gboolean use_advanced;
-        GtkWidget *clist;
         
 	use_advanced = gconf_client_get_bool (
 				panel_gconf_get_client (),
@@ -1127,7 +1158,6 @@ update_contents (GtkWidget *dialog)
         advanced_toggle = g_object_get_data (G_OBJECT (dialog),
 					     "advanced_toggle_label");
 
-        clist = g_object_get_data (G_OBJECT (dialog), "dentry_list");
         
         if (use_advanced) {
                 advanced = g_object_get_data (G_OBJECT (dialog), "advanced");
@@ -1155,6 +1185,8 @@ update_contents (GtkWidget *dialog)
                                       NULL);                
 
         } else {                
+		GtkWidget *tree_view;
+
                 advanced = g_object_get_data (G_OBJECT (dialog), "advanced");
                 
                 if (advanced && advanced->parent != NULL)
@@ -1166,7 +1198,8 @@ update_contents (GtkWidget *dialog)
                                       _("Allow typing in a command line instead of choosing an application from the list"),
                                       NULL);
 
-                gtk_widget_grab_focus (clist);
+		tree_view = g_object_get_data (G_OBJECT (dialog), "dentry_list");
+                gtk_widget_grab_focus (tree_view);
         }
 }
 
