@@ -87,55 +87,49 @@ launch_url (Launcher *launcher)
 
 
 static void
-launch (Launcher *launcher)
+launch_cb (GtkWidget *widget,
+	   Launcher  *launcher)
 {
 	GnomeDesktopItem *item;
-	GnomeDesktopItemType type;
 
-	g_return_if_fail(launcher != NULL);
-	g_return_if_fail(launcher->ditem != NULL);
+	g_return_if_fail (launcher != NULL);
+	g_return_if_fail (launcher->ditem != NULL);
 
 	item = launcher->ditem;
-	type = gnome_desktop_item_get_entry_type (item);
 
-	if (type == GNOME_DESKTOP_ITEM_TYPE_LINK) {
+	if (gnome_desktop_item_get_entry_type (item) == GNOME_DESKTOP_ITEM_TYPE_LINK)
 		launch_url (launcher);
-	} else {
+
+	else {
 		GError *error = NULL;
-		gnome_desktop_item_launch (item,
-					   NULL /* file_list */,
-					   0 /* flags */,
-					   &error);
-		if (error != NULL) {
+
+		gnome_desktop_item_launch (item, NULL, 0, &error);
+		if (error) {
 			panel_error_dialog ("cannot_launch_icon",
 					    _("<b>Cannot launch icon</b>\n\n"
-					      "Details: %s"),
-					    error->message);
+					      "Details: %s"), error->message);
 			g_clear_error (&error);
 		}
 	}
 	
-	if(global_config.drawer_auto_close) {
-		GtkWidget *parent =
-			PANEL_WIDGET(launcher->button->parent)->panel_parent;
-		g_return_if_fail(parent!=NULL);
-		if(DRAWER_IS_WIDGET(parent)) {
-			BasePWidget *basep = BASEP_WIDGET(parent);
-			GtkWidget *grandparent = PANEL_WIDGET(basep->panel)->master_widget->parent;
-			GtkWidget *grandparentw =
-				PANEL_WIDGET(grandparent)->panel_parent;
-			drawer_widget_close_drawer (DRAWER_WIDGET (parent),
-						    grandparentw);
+	if (global_config.drawer_auto_close) {
+		GtkWidget *parent;
+
+		parent = PANEL_WIDGET (launcher->button->parent)->panel_parent;
+		if (DRAWER_IS_WIDGET (parent)) {
+			GtkWidget *grandparent;
+
+			grandparent = PANEL_WIDGET (
+						PANEL_WIDGET (
+							BASEP_WIDGET (parent)->panel
+							     )->master_widget->parent
+						     )->panel_parent;
+			drawer_widget_close_drawer (
+					DRAWER_WIDGET (parent), grandparent);
 		}
 	}
 }
 
-static void
-launch_cb (GtkWidget *widget, gpointer data)
-{
-	launch (data);
-}
-
 static void
 drag_data_received_cb (GtkWidget        *widget,
 		       GdkDragContext   *context,
@@ -279,7 +273,6 @@ drag_data_get_cb (GtkWidget          *widget,
 		  guint               time,
 		  Launcher           *launcher)
 {
-	char *uri_list;
 	const char *location;
 	
 	g_return_if_fail (launcher != NULL);
@@ -287,25 +280,33 @@ drag_data_get_cb (GtkWidget          *widget,
 
 	location = gnome_desktop_item_get_location (launcher->ditem);
 
-	if (location == NULL)
+	if (!location) {
 		launcher_save (launcher);
+		location = gnome_desktop_item_get_location (launcher->ditem);
+	}
 
 	if (info == TARGET_URI_LIST) {
-		uri_list = g_strconcat ("file:", location, "\r\n", NULL);
+		if (location [0] != '/')
+			gtk_selection_data_set (
+				selection_data, selection_data->target, 8,
+				(guchar *)location, strlen (location));
+		else {
+			char *uri;
 
-		gtk_selection_data_set (selection_data,
-					selection_data->target, 8, (guchar *)uri_list,
-					strlen (uri_list));
-		g_free(uri_list);
-	} else if (info == TARGET_ICON_INTERNAL) {
+			uri = g_strconcat ("file://", location, "\r\n", NULL);
+
+			gtk_selection_data_set (
+				selection_data, selection_data->target, 8,
+				(guchar *)uri, strlen (uri));
+
+			g_free (uri);
+		}
+	} else if (info == TARGET_ICON_INTERNAL)
 		gtk_selection_data_set (selection_data,
 					selection_data->target, 8,
 					location, strlen (location));
-	}
 }
 
-
-
 static Launcher *
 create_launcher (const char *parameters, GnomeDesktopItem *ditem)
 {
@@ -327,7 +328,6 @@ create_launcher (const char *parameters, GnomeDesktopItem *ditem)
 							 0 /* flags */,
 							 NULL /* error */);
 
-		
 		if (ditem == NULL) {
 			gchar *entry;
 
@@ -380,30 +380,20 @@ create_launcher (const char *parameters, GnomeDesktopItem *ditem)
 	gtk_drag_dest_set (GTK_WIDGET (launcher->button),
 			   0, NULL, 0, 0);
 
-	g_signal_connect (G_OBJECT(launcher->button), "drag_data_get",
-			   G_CALLBACK(drag_data_get_cb),
-			   launcher);
-	g_signal_connect (G_OBJECT(launcher->button), "drag_data_received",
-			   G_CALLBACK(drag_data_received_cb),
-			   launcher);
-	g_signal_connect (G_OBJECT(launcher->button), "drag_motion",
-			   G_CALLBACK(drag_motion_cb),
-			   launcher);
-	g_signal_connect (G_OBJECT(launcher->button), "drag_drop",
-			   G_CALLBACK(drag_drop_cb),
-			   launcher);
-	g_signal_connect (G_OBJECT(launcher->button), "drag_leave",
-			   G_CALLBACK(drag_leave_cb),
-			   launcher);
-
-
-	g_signal_connect (G_OBJECT(launcher->button), "clicked",
-			    (GtkSignalFunc) launch_cb,
-			    launcher);
-	
-	g_signal_connect (G_OBJECT(launcher->button), "destroy",
-			    G_CALLBACK(destroy_launcher),
-			    launcher);
+	g_signal_connect (launcher->button, "drag_data_get",
+			   G_CALLBACK (drag_data_get_cb), launcher);
+	g_signal_connect (launcher->button, "drag_data_received",
+			   G_CALLBACK (drag_data_received_cb), launcher);
+	g_signal_connect (launcher->button, "drag_motion",
+			   G_CALLBACK (drag_motion_cb), launcher);
+	g_signal_connect (launcher->button, "drag_drop",
+			   G_CALLBACK (drag_drop_cb), launcher);
+	g_signal_connect (launcher->button, "drag_leave",
+			   G_CALLBACK (drag_leave_cb), launcher);
+	g_signal_connect (launcher->button, "clicked",
+			  G_CALLBACK (launch_cb), launcher);
+	g_signal_connect (launcher->button, "destroy",
+			  G_CALLBACK (destroy_launcher), launcher);
 
 	launcher->ditem = ditem;
 
@@ -427,12 +417,16 @@ setup_button (Launcher *launcher)
 						       GNOME_DESKTOP_ITEM_COMMENT);
 
 	/* Setup tooltip */
-	if (comment != NULL)
+	if ( ! string_empty (comment))
 		str = g_strdup_printf ("%s\n%s", name, comment);
 	else
 		str = g_strdup (name);
 	gtk_tooltips_set_tip (panel_tooltips, launcher->button,
 			      str, NULL);
+
+	/* Setup accessible name */
+	add_atk_name_desc (launcher->button, str, NULL);
+
 	g_free (str);
 
 	/* Setup text */
@@ -671,8 +665,8 @@ launcher_load_from_gconf (PanelWidget *panel_widget,
 	const char  *temp_key;
 	char        *launcher_location;
 
-	g_return_if_fail (panel_widget);
-	g_return_if_fail (gconf_key);
+	g_return_if_fail (panel_widget != NULL);
+	g_return_if_fail (gconf_key != NULL);
 
 	client  = panel_gconf_get_client ();
 	profile = panel_gconf_get_profile ();
@@ -973,7 +967,7 @@ launcher_save (Launcher *launcher)
 	g_return_if_fail (launcher != NULL);
 	g_return_if_fail (launcher->ditem != NULL);
 
-	if (gnome_desktop_item_get_location (launcher->ditem) == NULL)
+	if (!gnome_desktop_item_get_location (launcher->ditem))
 		gnome_desktop_item_set_location (launcher->ditem,
 						 launcher_get_unique_uri ());
 
