@@ -63,8 +63,6 @@ ditem_properties_apply_timeout (gpointer data)
 		g_object_set_data (G_OBJECT (dedit), "SavingError", NULL);
 	}
 
-	gnome_desktop_item_unref (ditem);
-
 	return FALSE;
 }
 
@@ -310,7 +308,7 @@ panel_edit_direntry (const char *dir, const char *dir_name)
 
 	set_ditem_sensitive (GTK_DIALOG (dialog),
 			     GNOME_DITEM_EDIT (dedit),
-			     NULL /* loc */, dir);
+			     dirfile, NULL);
 
 	g_signal_connect (G_OBJECT (dedit), "changed",
 			    G_CALLBACK (ditem_properties_changed),
@@ -320,14 +318,6 @@ panel_edit_direntry (const char *dir, const char *dir_name)
 			    G_CALLBACK (ditem_properties_close),
 			    dedit);
 
-	/* YAIKES, the problem here is that the notebook will attempt
-	 * to destroy the dedit, so if we unref it in the close handler,
-	 * it will be finalized by the time the notebook will destroy it,
-	 * dedit is just a horrible thing */
-	g_signal_connect (G_OBJECT (dedit), "destroy",
-			  G_CALLBACK (g_object_unref),
-			  NULL);
-
 	g_object_set_data (G_OBJECT (dialog), "GnomeDItemEdit", dedit);
 
 	if (ditem != NULL) {
@@ -450,7 +440,7 @@ really_add_new_menu_item (GtkWidget *d, int response, gpointer data)
 	}
 
 	/* assume we are making a new file */
-	name = g_strdup (gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_NAME));
+	name = g_strdup (gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME));
 
 	validate_for_filename (name);
 
