@@ -33,9 +33,6 @@
 #include "panel.h"
 #include "multiscreen-stuff.h"
 
-/*
- * FIXME
- */
 #include "nothing.cP"
 
 static void config_apply (PerPanelConfig *ppc);
@@ -595,15 +592,14 @@ screen_set (GtkWidget *widget, gpointer data)
 }
 
 static GtkWidget *
-make_misc_widget (PerPanelConfig *ppc, gboolean avoiding)
+make_misc_widget (PerPanelConfig *ppc)
 {
 	GtkWidget *frame;
 	GtkWidget *box, *hbox;
 	GtkWidget *button, *label;
 	GtkObject *range;
 
-	if ( ! avoiding &&
-	    multiscreen_screens () <= 1)
+	if (multiscreen_screens () <= 1)
 		return NULL;
 
 	frame = gtk_frame_new (_("Miscellaneous"));
@@ -613,25 +609,23 @@ make_misc_widget (PerPanelConfig *ppc, gboolean avoiding)
 	gtk_container_set_border_width (GTK_CONTAINER (box), GNOME_PAD_SMALL);
 	gtk_container_add (GTK_CONTAINER (frame), box);
 	
-	if (multiscreen_screens () > 1) {
-		hbox = gtk_hbox_new (FALSE, 0);
-		gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);
+	hbox = gtk_hbox_new (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (box), hbox, FALSE, FALSE, 0);
 
-		label = gtk_label_new (_("Current screen:"));
-		gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+	label = gtk_label_new (_("Current screen:"));
+	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
-		range = gtk_adjustment_new (ppc->screen,
-					    0, multiscreen_screens () - 1,
-					    1, 1, 1);
-		ppc->screen_spin = button =
-			gtk_spin_button_new (GTK_ADJUSTMENT (range), 1, 0);
-		gtk_widget_set_size_request (GTK_WIDGET (button), 65, -1);
-		g_object_set_data (G_OBJECT (button), "PerPanelConfig", ppc);
-		gtk_spin_button_set_value (GTK_SPIN_BUTTON (button), ppc->screen);
-		g_signal_connect (G_OBJECT (button), "changed",
-				  G_CALLBACK (screen_set), NULL);
-		gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
-	}
+	range = gtk_adjustment_new (ppc->screen,
+				    0, multiscreen_screens () - 1,
+				    1, 1, 1);
+
+	ppc->screen_spin = button =
+		gtk_spin_button_new (GTK_ADJUSTMENT (range), 1, 0);
+	gtk_widget_set_size_request (GTK_WIDGET (button), 65, -1);
+	g_object_set_data (G_OBJECT (button), "PerPanelConfig", ppc);
+	gtk_spin_button_set_value (GTK_SPIN_BUTTON (button), ppc->screen);
+	g_signal_connect (button, "changed", G_CALLBACK (screen_set), NULL);
+	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 
 	return frame;
 }
@@ -670,14 +664,15 @@ make_position_widget (PerPanelConfig *ppc, int aligns)
 	GtkWidget *pos_box;
 	GtkWidget *table;
 	GtkWidget *w = NULL;
+	GtkWidget *label;
 	int align;
 
 	pos_box = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
 	gtk_container_set_border_width (GTK_CONTAINER (pos_box),
 					GNOME_PAD_SMALL);
 
-	w = gtk_label_new (_("Panel Position"));
-	gtk_box_pack_start (GTK_BOX (pos_box), w, FALSE, FALSE, 0);
+	label = gtk_label_new (_("Panel Position"));
+	gtk_box_pack_start (GTK_BOX (pos_box), label, FALSE, FALSE, 0);
 
 	w = gtk_alignment_new (0.5, 0.5, 0, 0);	
 	gtk_box_pack_start (GTK_BOX (pos_box), w, FALSE, FALSE, 0);
@@ -685,6 +680,8 @@ make_position_widget (PerPanelConfig *ppc, int aligns)
 	table = gtk_table_new (2 + aligns, 2 + aligns, FALSE);
 	gtk_container_add (GTK_CONTAINER (w), table);
 
+	set_relation (table, GTK_LABEL (label), 1);
+
 	w = NULL;
 
 	/* LEFT */
@@ -815,7 +812,7 @@ edge_notebook_page (PerPanelConfig *ppc)
 	w = make_hidebuttons_widget (ppc);
 	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
 
-	w = make_misc_widget (ppc, TRUE);
+	w = make_misc_widget (ppc);
 	if (w != NULL)
 		gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
 	
@@ -850,7 +847,7 @@ aligned_notebook_page (PerPanelConfig *ppc)
 	w = make_hidebuttons_widget (ppc);
 	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
 
-	w = make_misc_widget (ppc, TRUE);
+	w = make_misc_widget (ppc);
 	if (w != NULL)
 		gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
 	
@@ -953,11 +950,12 @@ floating_notebook_page (PerPanelConfig *ppc)
 	gtk_widget_set_size_request (GTK_WIDGET (button), 65, -1);
 	g_object_set_data (G_OBJECT (button), "PerPanelConfig", ppc);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (button), ppc->x);
-	g_signal_connect (G_OBJECT (button), "changed",
-			  G_CALLBACK (floating_set_xy),
-			  &ppc->x);
+	g_signal_connect (button, "value-changed",
+			  G_CALLBACK (floating_set_xy), &ppc->x);
 	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 
+	set_relation (ppc->x_spin, GTK_LABEL (label), 1);
+
 	label = gtk_label_new (_("Y"));
 	gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 	
@@ -967,15 +965,16 @@ floating_notebook_page (PerPanelConfig *ppc)
 	gtk_widget_set_size_request (GTK_WIDGET (button), 65, -1);
 	g_object_set_data (G_OBJECT (button), "PerPanelConfig", ppc);
 	gtk_spin_button_set_value (GTK_SPIN_BUTTON (button), ppc->y);
-	g_signal_connect (G_OBJECT (button), "changed",
-			  G_CALLBACK (floating_set_xy),
-			  &ppc->y);
+	g_signal_connect (button, "value-changed",
+			  G_CALLBACK (floating_set_xy), &ppc->y);
 	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 
+	set_relation (ppc->y_spin, GTK_LABEL (label), 1);
+
 	w = make_hidebuttons_widget (ppc);
 	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
 
-	w = make_misc_widget (ppc, FALSE);
+	w = make_misc_widget (ppc);
 	if (w != NULL)
 		gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
 	
@@ -1050,10 +1049,12 @@ sliding_notebook_page (PerPanelConfig *ppc)
 			  G_CALLBACK (sliding_set_offset), ppc);
 	gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
 
+	set_relation (ppc->offset_spin, GTK_LABEL (l), 1);
+
 	w = make_hidebuttons_widget (ppc);
 	gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
 	
-	w = make_misc_widget (ppc, TRUE);
+	w = make_misc_widget (ppc);
 	if (w != NULL)
 		gtk_box_pack_start (GTK_BOX (vbox), w, FALSE, FALSE, 0);
 	
@@ -1074,18 +1075,6 @@ size_set_size (GtkWidget *widget, gpointer data)
 	panel_config_register_changes (ppc);
 }
 
-/* XXX: until this is fixed in GTK+ */
-static void
-activate_proper_item (GtkMenuShell *menu)
-{
-#ifdef FIXME
-	GtkWidget *active;
-	active = gtk_menu_shell_get_active (menu);
-	if (active != NULL)
-		gtk_menu_item_activate (GTK_MENU_ITEM (active));
-#endif
-}
-
 GtkWidget *
 make_size_widget (PerPanelConfig *ppc)
 {
@@ -1094,6 +1083,7 @@ make_size_widget (PerPanelConfig *ppc)
 	GtkWidget *menu;
 	GtkWidget *menuitem;
 	GtkWidget *w;
+	GtkWidget *label;
 	gchar *s;
 	int i;
 
@@ -1103,17 +1093,15 @@ make_size_widget (PerPanelConfig *ppc)
 	box = gtk_hbox_new (FALSE, GNOME_PAD_SMALL);
 	gtk_box_pack_start (GTK_BOX (vbox), box, FALSE, FALSE, 0);
 
+	label = gtk_label_new (_("Panel size:"));
 	gtk_box_pack_start (GTK_BOX (box),
-			    gtk_label_new (_("Panel size:")),
+			    label,
 			    FALSE, FALSE, 0);
 
 	
 	menu = gtk_menu_new ();
-	g_signal_connect (G_OBJECT (menu), "deactivate", 
-			  G_CALLBACK (activate_proper_item), 
-			  NULL);
 
-	menuitem = gtk_menu_item_new_with_label (_("XX Small Tiny (12 pixels)"));
+	menuitem = gtk_menu_item_new_with_label (_("XX Small (12 pixels)"));
 	gtk_widget_show(menuitem);
 	gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
 	g_object_set_data (G_OBJECT (menuitem), "PerPanelConfig", ppc);
@@ -1174,6 +1162,8 @@ make_size_widget (PerPanelConfig *ppc)
 	
 	gtk_box_pack_start (GTK_BOX (box), ppc->size_menu,
 			    FALSE, FALSE, 0);
+
+	set_relation (ppc->size_menu, GTK_LABEL (label), 1);
 	
 	s = _("Note: The panel will size itself to the\n"
 	      "largest applet in the panel, and that\n"
@@ -1301,6 +1291,7 @@ background_page (PerPanelConfig *ppc)
 	GtkWidget *box, *f, *t;
 	GtkWidget *vbox, *noscale, *fit, *stretch;
 	GtkWidget *w, *m;
+	GtkWidget *label;
 
 	vbox = gtk_vbox_new (FALSE, GNOME_PAD_SMALL);
 	gtk_container_set_border_width(GTK_CONTAINER (vbox), GNOME_PAD_SMALL);
@@ -1313,9 +1304,6 @@ background_page (PerPanelConfig *ppc)
 
 	/*background type option menu*/
 	m = gtk_menu_new ();
-	g_signal_connect (G_OBJECT (m), "deactivate", 
-			  G_CALLBACK (activate_proper_item), 
-			  NULL);
 	ppc->non = gtk_menu_item_new_with_label (_("Standard"));
 	g_object_set_data (G_OBJECT (ppc->non), "PerPanelConfig", ppc);
 	gtk_widget_show (ppc->non);
@@ -1333,6 +1321,7 @@ background_page (PerPanelConfig *ppc)
 	gtk_option_menu_set_menu (GTK_OPTION_MENU (ppc->back_om), m);
 	gtk_box_pack_start (GTK_BOX (box), ppc->back_om, FALSE, FALSE, 0);
 
+	set_relation (ppc->back_om, GTK_LABEL (w), 1);
 
 	/*color frame*/
 	f = gtk_frame_new (_("Color"));
@@ -1347,7 +1336,8 @@ background_page (PerPanelConfig *ppc)
 	gtk_container_set_border_width(GTK_CONTAINER (box), GNOME_PAD_SMALL);
 	gtk_container_add (GTK_CONTAINER (f), box);
 
-	gtk_box_pack_start (GTK_BOX (box), gtk_label_new (_("Color to use:")),
+	label = gtk_label_new (_("Color to use:"));
+	gtk_box_pack_start (GTK_BOX (box), label,
 			    FALSE, FALSE, GNOME_PAD_SMALL);
 
 	ppc->backsel = gnome_color_picker_new();
@@ -1359,6 +1349,7 @@ background_page (PerPanelConfig *ppc)
 				   ppc->back_color.green,
 				   ppc->back_color.blue,
 				   65535);
+	set_relation (ppc->backsel, GTK_LABEL (label), 1);
 
 	gtk_box_pack_start (GTK_BOX (box), ppc->backsel, FALSE, FALSE, 0);
 
