@@ -34,7 +34,6 @@
 
 #include "applet.h"
 #include "nothing.h"
-#include "panel-config-global.h"
 
 extern GlobalConfig global_config;
 
@@ -45,12 +44,10 @@ panel_show_help (const char *doc_name, const char *linkid)
 {
 	GError *error = NULL;
 
-	if ( ! gnome_help_display (doc_name,
-				   linkid,
-				   &error)) {
+	if (!gnome_help_display_desktop (NULL, "panel", doc_name, linkid, &error)) {
 		panel_error_dialog ("cannot_show_help",
 				    _("<b>Cannot display help document</b>\n\n"
-				      "Details: %s"), error->message);
+				      "Details: %s"), error != NULL ? error->message : "");
 		g_clear_error (&error);
 	}
 }
@@ -208,6 +205,7 @@ create_text_entry(GtkWidget *table,
 			 GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			 GTK_FILL | GTK_SHRINK,
 			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
+	set_relation (entry, GTK_LABEL (wlabel), 1);
 
 	if(func) {
 		g_object_set_data (G_OBJECT (t), "update_function", func);
@@ -246,6 +244,7 @@ create_icon_entry(GtkWidget *table,
 	if (text != NULL)
 		gnome_icon_entry_set_filename (GNOME_ICON_ENTRY(entry), text);
 
+	set_relation (entry, GTK_LABEL (wlabel), 1);
 	gtk_table_attach(GTK_TABLE(table), entry,
 			 cols, cole, 1, 2,
 			 GTK_EXPAND | GTK_FILL | GTK_SHRINK,
@@ -570,24 +569,21 @@ panel_set_frame_colors (PanelWidget *panel, GtkWidget *frame,
 }
 
 gboolean
-convert_string_to_keysym_state(const char *string,
-			       guint *keysym,
-			       GdkModifierType *state)
+panel_parse_accelerator (GlobalConfigKey *key)
 {
-	g_return_val_if_fail (keysym != NULL, FALSE);
-	g_return_val_if_fail (state != NULL, FALSE);
+	g_return_val_if_fail (key != NULL, FALSE);
 	
-	*state = 0;
-	*keysym = 0;
+	key->keysym = 0;
+	key->state = 0;
 
-	if (string_empty (string) ||
-	    strcmp (string, "Disabled") == 0 ||
-	    strcmp (string, _("Disabled")) == 0)
+	if (!key->str || !key->str [0] ||
+	    !strcmp (key->str, "Disabled") ||
+	    !strcmp (key->str, _("Disabled")))
 		return FALSE;
 
-	gtk_accelerator_parse (string, keysym, state);
+	gtk_accelerator_parse (key->str, &key->keysym, &key->state);
 
-	if (*keysym == 0)
+	if (!key->keysym)
 		return FALSE;
 
 	return TRUE;
@@ -967,48 +963,65 @@ panel_pixmap_discovery (const char *name, gboolean fallback)
 	return pixmap;
 }
 
+static GtkWidget *
+stretch_widget_calc_geometry (GtkWidget *widget,
+			      int       *x,
+			      int       *y,
+			      int       *w,
+			      int       *h)
+{
+	PanelStretchFlags  flags;
+	GtkWidget         *toplevel;
+
+	g_assert (x && y && w && h);
+
+	flags = GPOINTER_TO_INT (
+			g_object_get_data (G_OBJECT (widget) , "stretch-flags"));
+	toplevel = gtk_widget_get_toplevel (widget);
+
+	if (!toplevel || !GTK_WIDGET_REALIZED (toplevel))
+		return NULL;
+	
+	gtk_widget_translate_coordinates (widget, toplevel, 0, 0, x, y);
+
+	*w = widget->allocation.width;
+	*h = widget->allocation.height;
+
+	if (flags & PANEL_STRETCH_TOP) {
+		*h += *y;
+		*y  = 0;
+	}
+
+	if (flags & PANEL_STRETCH_LEFT) {
+		*w += *x;
+		*x  = 0;
+	}
+
+	if (flags & PANEL_STRETCH_BOTTOM)
+		*h = toplevel->allocation.height - *y;
+
+	if (flags & PANEL_STRETCH_RIGHT)
+		*w = toplevel->allocation.width - *x;
+
+	return toplevel;
+}
+
 static void
 stretch_widget_realize (GtkWidget *widget)
 {
-	GdkWindowAttr attributes = {0};
-	int attributes_mask;
-	GtkWidget *toplevel;
-	int x,y,w,h;
-	PanelStretchFlags flags;
-	GdkWindow *eventwin = g_object_get_data (G_OBJECT (widget),
-						 "StrechEventWindow");
-	flags = GPOINTER_TO_INT (
-			g_object_get_data (G_OBJECT (widget) , "stretch-flags"));
+	GdkWindowAttr  attributes = { 0 };
+	int            attributes_mask;
+	int            x, y, w, h;
+	GdkWindow     *eventwin;
+	GtkWidget     *toplevel;
 
+	eventwin = g_object_get_data (G_OBJECT (widget), "StrechEventWindow");
 	if (eventwin != NULL)
 		gdk_window_destroy (eventwin);
 
-	toplevel = gtk_widget_get_toplevel (widget);
-
-	if (toplevel == NULL ||
-	    ! GTK_WIDGET_REALIZED (toplevel))
+	toplevel = stretch_widget_calc_geometry (widget, &x, &y, &w, &h);
+	if (!toplevel)
 		return;
-	
-	gtk_widget_translate_coordinates (widget /* src_widget */,
-					  toplevel /* dest_widget */,
-					  0, 0 /* src */,
-					  &x, &y /* dest */);
-
-	w = widget->allocation.width;
-	h = widget->allocation.height;
-
-	if (flags & PANEL_STRETCH_TOP) {
-		h += y;
-		y = 0;
-	}
-	if (flags & PANEL_STRETCH_LEFT) {
-		w += x;
-		x = 0;
-	}
-	if (flags & PANEL_STRETCH_BOTTOM)
-		h = toplevel->allocation.height - y;
-	if (flags & PANEL_STRETCH_RIGHT)
-		w = toplevel->allocation.width - x;
 
 	attributes.window_type = GDK_WINDOW_CHILD;
 	attributes.x = x;
@@ -1097,11 +1110,23 @@ stretch_widget_unmap (GtkWidget *widget)
 
 
 static void
-stretch_widget_size_allocate (GtkWidget *widget, GtkAllocation *allocation)
+stretch_widget_size_allocate (GtkWidget     *widget,
+			      GtkAllocation *allocation)
 {
-	GdkWindow *eventwin = g_object_get_data (G_OBJECT (widget),
-						 "StrechEventWindow");
-	if (eventwin == NULL)
+	GdkWindow *eventwin;
+	int        x, y, w, h;
+	int        old_x, old_y, old_w, old_h, old_d;
+
+	eventwin = g_object_get_data (G_OBJECT (widget), "StrechEventWindow");
+	if (!eventwin)
+		return;
+
+	if (!stretch_widget_calc_geometry (widget, &x, &y, &w, &h))
+		return;
+
+	gdk_window_get_geometry (eventwin, &old_x, &old_y, &old_w, &old_h, &old_d);
+
+	if ((x == old_x) && (y == old_y) && (w == old_w) && (h == old_h))
 		return;
 
 	/* somewhat evil */
@@ -1242,6 +1267,115 @@ panel_g_slist_deep_free	(GSList *list)
 	g_slist_free (list);
 }
 
+static gboolean
+internal_panel_is_uri_writable (const char *uri, gboolean recurse)
+{
+	GnomeVFSFileInfo *info = gnome_vfs_file_info_new ();
+
+	if (gnome_vfs_get_file_info
+	    (uri, info, GNOME_VFS_FILE_INFO_DEFAULT) != GNOME_VFS_OK) {
+		char *dir;
+		gboolean ret;
+
+		gnome_vfs_file_info_unref (info);
+
+		if ( ! recurse)
+			return FALSE;
+
+		dir = g_path_get_dirname (uri);
+		ret = internal_panel_is_uri_writable (dir, FALSE);
+		g_free (dir);
+
+		return ret;
+	}
+
+	if ( ! (info->valid_fields & GNOME_VFS_FILE_INFO_FIELDS_PERMISSIONS)) {
+		gnome_vfs_file_info_unref (info);
+		/* assume writable, if no permissions */
+		return TRUE;
+	} 
+
+	if (info->permissions & GNOME_VFS_PERM_OTHER_WRITE) {
+		gnome_vfs_file_info_unref (info);
+		return TRUE;
+	}
+
+	if (info->gid == getgid () &&
+	    info->permissions & GNOME_VFS_PERM_GROUP_WRITE) {
+		gnome_vfs_file_info_unref (info);
+		return TRUE;
+	}
+
+	if (info->uid == getuid () &&
+	    info->permissions & GNOME_VFS_PERM_USER_WRITE) {
+		gnome_vfs_file_info_unref (info);
+		return TRUE;
+	}
+
+	if (info->gid == getgid () &&
+	    info->permissions & GNOME_VFS_PERM_GROUP_WRITE) {
+		gnome_vfs_file_info_unref (info);
+		return TRUE;
+	}
+
+	if (info->permissions & GNOME_VFS_PERM_GROUP_WRITE) {
+		gid_t *groups;
+		int i, n;
+
+		/* get size */
+		n = getgroups (0, NULL);
+
+		if (n == 0) {
+			/* no more groups */
+			gnome_vfs_file_info_unref (info);
+			return FALSE;
+		}
+
+		groups = g_new0 (gid_t, n);
+
+		n = getgroups (n, groups);
+		for (i = 0; i < n; i++) {
+			if (info->gid == groups[i]) {
+				/* ok */
+				gnome_vfs_file_info_unref (info);
+				return TRUE;
+			}
+		}
+	}
+
+	/* no more perimission stuff to try */
+	gnome_vfs_file_info_unref (info);
+	return FALSE;
+}
+
+gboolean
+panel_is_uri_writable (const char *uri)
+{
+	g_return_val_if_fail (uri != NULL, FALSE);
+
+	return internal_panel_is_uri_writable (uri, TRUE /* recurse */);
+}
+
+gboolean
+panel_uri_exists (const char *uri)
+{
+	GnomeVFSURI *suri;
+	gboolean ret;
+
+	g_return_val_if_fail (uri != NULL, FALSE);
+
+	suri = gnome_vfs_uri_new (uri);
+
+	/* eek */
+	g_return_val_if_fail (suri != NULL, FALSE);
+
+	ret = gnome_vfs_uri_exists (suri);
+
+	gnome_vfs_uri_unref (suri);
+
+	return ret;
+}
+
 #if 0
 /* hmmm, we need to use this I think to add a battery applet
  *  -George */
