@@ -107,6 +107,7 @@ struct _PanelWidget
 						   was set, this is used
 						   for tiling onto the
 						   background */
+	GdkEventKey		*key_event;
 };
 
 struct _PanelWidgetClass
@@ -230,6 +231,10 @@ void            panel_widget_focus              (PanelWidget *panel);
 
 PanelOrient     panel_widget_get_applet_orient  (PanelWidget *panel);
 
+void            panel_widget_save_key_event	(PanelWidget	*panel,
+                                                 GdkEventKey	*key_event);
+
+
 extern gboolean panel_applet_in_drag;
 
 G_END_DECLS
