@@ -29,7 +29,10 @@
 #include <unistd.h>
 #include <stdlib.h>
 #include <string.h>
+#include <gdk/gdk.h>
 #include <gdk/gdkx.h>
+#include <gdk/gdkkeysyms.h>
+#include <gtk/gtkbindings.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <bonobo/bonobo-main.h>
 #include <bonobo/bonobo-types.h>
@@ -68,6 +71,8 @@ struct _PanelAppletPrivate {
 
         int                        *size_hints;
         int                         size_hints_len;
+
+	gboolean		    moving_focus_out;
 };
 
 static GObjectClass *parent_class;
@@ -76,6 +81,7 @@ enum {
         CHANGE_ORIENT,
         CHANGE_SIZE,
         CHANGE_BACKGROUND,
+	MOVE_FOCUS_OUT_OF_APPLET,
         SAVE_YOURSELF,
         LAST_SIGNAL
 };
@@ -445,14 +451,12 @@ panel_applet_button_press (GtkWidget      *widget,
 	return FALSE;
 }
 
-static gboolean
-panel_applet_popup_menu (GtkWidget *widget)
+gboolean
+_panel_applet_popup_menu (PanelApplet *applet)
 {
-	PanelApplet *applet = PANEL_APPLET (widget);
-
 	bonobo_control_do_popup_full (applet->priv->control, NULL, NULL,
-				      panel_applet_menu_position, widget,
-				      3,
+				      panel_applet_menu_position,
+				      GTK_WIDGET (applet), 3,
 				      GDK_CURRENT_TIME);
 	return TRUE;
 	
@@ -485,6 +489,13 @@ panel_applet_focus (GtkWidget        *widget,
 {
 	gboolean ret;
 	GtkWidget *previous_focus_child;
+	PanelApplet *applet;
+
+	g_return_val_if_fail (PANEL_IS_APPLET (widget), FALSE);
+
+	applet = PANEL_APPLET (widget);
+	if (applet->priv->moving_focus_out)
+		return FALSE;
 
 	previous_focus_child = GTK_CONTAINER (widget)->focus_child;
 	ret = GTK_WIDGET_CLASS (parent_class)->focus (widget, dir);
@@ -699,8 +710,6 @@ panel_applet_get_prop (BonoboPropertyBag *sack,
 		       gpointer           user_data)
 {
 	PanelApplet *applet = PANEL_APPLET (user_data);
-	CORBA_sequence_CORBA_long *seq;
-	int i;
 
 	switch (arg_id) {
 	case PROPERTY_ORIENT_IDX:
@@ -715,15 +724,19 @@ panel_applet_get_prop (BonoboPropertyBag *sack,
 	case PROPERTY_FLAGS_IDX:
 		BONOBO_ARG_SET_SHORT (arg, applet->priv->flags);
 		break;
-	case PROPERTY_SIZE_HINTS_IDX:
+	case PROPERTY_SIZE_HINTS_IDX: {
+		CORBA_sequence_CORBA_long *seq;
+		int                        i;
+
 		seq = arg->_value;
 
-		seq->_buffer = CORBA_sequence_CORBA_long_allocbuf (applet->priv->size_hints_len);
-		seq->_maximum = applet->priv->size_hints_len;
-		seq->_length = applet->priv->size_hints_len;
+		seq->_length  = seq->_maximum = applet->priv->size_hints_len;
+		seq->_buffer  = CORBA_sequence_CORBA_long_allocbuf (seq->_length);
+		seq->_release = CORBA_TRUE;
 		
 		for (i = 0; i < applet->priv->size_hints_len; i++)
 			seq->_buffer [i] = applet->priv->size_hints[i];
+		}
 		break;
 	default:
 		g_assert_not_reached ();
@@ -985,15 +998,49 @@ panel_applet_item_handler_get_object (BonoboItemHandler *handler,
 	return bonobo_object_dup_ref (BONOBO_OBJREF (applet->priv->control), ev);
 }
 
+static void
+panel_applet_move_focus_out_of_applet (PanelApplet      *applet,
+				       GtkDirectionType  dir)
+{
+	GtkWidget *toplevel;
+
+	applet->priv->moving_focus_out = TRUE;
+	toplevel = gtk_widget_get_toplevel (GTK_WIDGET (applet));
+	g_return_if_fail (toplevel);
+
+	gtk_widget_child_focus (toplevel, dir);
+	applet->priv->moving_focus_out = FALSE;
+}
+
+static void
+add_tab_bindings (GtkBindingSet   *binding_set,
+		  GdkModifierType  modifiers,
+		  GtkDirectionType direction)
+{
+	gtk_binding_entry_add_signal (binding_set, GDK_Tab, modifiers,
+				      "move_focus_out_of_applet", 1,
+				      GTK_TYPE_DIRECTION_TYPE, direction);	
+	gtk_binding_entry_add_signal (binding_set, GDK_KP_Tab, modifiers,
+				      "move_focus_out_of_applet", 1,
+				      GTK_TYPE_DIRECTION_TYPE, direction);	
+	gtk_binding_entry_add_signal (binding_set, GDK_ISO_Left_Tab, modifiers,
+				      "move_focus_out_of_applet", 1,
+				      GTK_TYPE_DIRECTION_TYPE, direction);	
+}
+
 static void
 panel_applet_class_init (PanelAppletClass *klass,
 			 gpointer          dummy)
 {
 	GObjectClass   *gobject_class = (GObjectClass *) klass;
+	GtkObjectClass *object_class = (GtkObjectClass *) klass;
 	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
+	GtkBindingSet *binding_set;
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	klass->move_focus_out_of_applet = panel_applet_move_focus_out_of_applet;
+
 	widget_class->button_press_event = panel_applet_button_press;
 	widget_class->expose_event = panel_applet_expose;
 	widget_class->focus = panel_applet_focus;
@@ -1037,6 +1084,24 @@ panel_applet_class_init (PanelAppletClass *klass,
 			      PANEL_TYPE_PANEL_APPLET_BACKGROUND_TYPE,
 			      G_TYPE_POINTER,
 			      GDK_TYPE_PIXMAP);
+
+	panel_applet_signals [MOVE_FOCUS_OUT_OF_APPLET] =
+                g_signal_new ("move_focus_out_of_applet",
+                              G_TYPE_FROM_CLASS (klass),
+                              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+                              G_STRUCT_OFFSET (PanelAppletClass, move_focus_out_of_applet),
+                              NULL,
+			      NULL,
+                              panel_applet_marshal_VOID__ENUM,
+                              G_TYPE_NONE,
+			      1,
+			      GTK_TYPE_DIRECTION_TYPE);
+
+	binding_set = gtk_binding_set_by_class (object_class);
+	add_tab_bindings (binding_set, 0, GTK_DIR_TAB_FORWARD);
+	add_tab_bindings (binding_set, GDK_SHIFT_MASK, GTK_DIR_TAB_BACKWARD);
+	add_tab_bindings (binding_set, GDK_CONTROL_MASK, GTK_DIR_TAB_FORWARD);
+	add_tab_bindings (binding_set, GDK_CONTROL_MASK | GDK_SHIFT_MASK, GTK_DIR_TAB_BACKWARD);
 }
 
 static void
@@ -1051,6 +1116,8 @@ panel_applet_instance_init (PanelApplet      *applet,
 	applet->priv->orient       = PANEL_APPLET_ORIENT_UP;
 	applet->priv->size         = GNOME_Vertigo_PANEL_MEDIUM;
 
+	applet->priv->moving_focus_out = FALSE;
+
 	gtk_widget_set_events (GTK_WIDGET (applet), 
 			       GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
 }
@@ -1112,7 +1179,7 @@ panel_applet_setup (PanelApplet *applet)
 				     BONOBO_OBJECT (priv->item_handler));
 
 	g_signal_connect (applet, "popup_menu",
-			  G_CALLBACK (panel_applet_popup_menu), NULL);
+			  G_CALLBACK (_panel_applet_popup_menu), NULL);
 }
 
 /**
